//To compile:
//1) Compile calctsr.c in BINARY mode (requires flag in tigcc)
//2) Convert output to hex with ttbin2hex
//3) Copy and paste hex into install.c
//4) Compile install.c to tsrcalc.89z
#include <tigcclib.h>
#define MAGIC_TSR 0x64535231  // magic marker for TSR program ("TSR1")

int _ti89;

#define SIZE_OF_PRG (sample_prg[0] * 256 + sample_prg[1] + 2)

unsigned char sample_prg[]={
#include "C:\program files\tigcc\calctsr.hex"
};

int InstallTSR(unsigned char* signature) {
    unsigned char* mem = HeapAllocPtr(SIZE_OF_PRG+12); // +12 for signature
    
    //---------------------------------------------------    
    // in case of a "cannot allocate enough memory" error
    // we will immediately return
    //---------------------------------------------------
    if (!mem) return 0;
    
    //-----------------------------------------------------
    // write MAGIC_TSR (4 bytes) at the beginning of memory
    //-----------------------------------------------------
    *mem++ = (MAGIC_TSR >> 24) & 0xff;
    *mem++ = (MAGIC_TSR >> 16) & 0xff;
    *mem++ = (MAGIC_TSR >>  8) & 0xff;
    *mem++ = (MAGIC_TSR      ) & 0xff;

    //-----------------------------------------------------
    // copy afterwards the 8 bytes signature to memory
    //-----------------------------------------------------
    memcpy(mem,signature,8);
    mem+=8;
    
    //-----------------------------------------------------
    // ... and the program itself from the sample_prg array
    //-----------------------------------------------------
    memcpy(mem,sample_prg,SIZE_OF_PRG);
    
    //--------------------------------------------------------
    // relocate program, bypass some protections introduced in 
    // AMS 2.xx and execute _main routine of TSR
    //--------------------------------------------------------
    EX_patch(mem + 0x40002, mem + SIZE_OF_PRG - 2 + 3 - 2);  
    enter_ghost_space();  
    ASM_call(mem + 0x40002);
    return 1;
}

void RemoveTSR(unsigned char* tsr_address) {
    //------------------------------------------------
    // bypass some protections introduced in 
    // AMS 2.xx and execute _main routine of TSR again
    //------------------------------------------------
    enter_ghost_space();  
    ASM_call(tsr_address + 12 + 0x40002);
    //--------------------------------------------------
    // fill memory holding signature with zeros and free
    // the buffer allocated by InstallTSR()
    //--------------------------------------------------
    memset(tsr_address,0,12); 
    HeapFreePtr(tsr_address); 
}


//=============================================================================
// try to find TSR on high heap
//
// an installed TSR starts with MAGIC_TSR (4 bytes) followed by an 8 byte
// program signature
//
// the complete memory is scanned backwards starting at the address returned
// by HeapEnd() until we'll reach address 0 (or we'll find the marker)
//
// returns address of already installed TSR or NULL
//=============================================================================
unsigned char* FindTSR(unsigned char* signature) {
    unsigned char* end = HeapEnd();
    end-=1000;
    while (end>1000) {
        if (*end     == ((MAGIC_TSR >> 24) & 0xff) &&
            *(end+1) == ((MAGIC_TSR >> 16) & 0xff) &&
            *(end+2) == ((MAGIC_TSR >>  8) & 0xff) &&
            *(end+3) == ((MAGIC_TSR      ) & 0xff) &&
            !strncmp(end+4,signature,8))
        {
            // we have found the TSR ....
            return end;
        }    
        end-=1;
    }
    
    //-----------------------------------------------------
    // if we'll come here we haven't found an installed TSR
    //-----------------------------------------------------
    return NULL;
}


//=============================================================================
// as usual: a main entry point ...
//=============================================================================
void _main(void) { 
    unsigned char  signature[8] = {'t','s','r','c','a','l','c','u'};
    unsigned char* tsr_address;

    //----------------------------------------------------
    // try to find an already installed version of the TSR
    //----------------------------------------------------
    tsr_address = FindTSR(signature);
    
    //-------------------------------------------
    // if we have found the signature on the heap
    // the TSR is already installed -> remove it
    //-------------------------------------------
    if (tsr_address) {
        RemoveTSR(tsr_address);
        ST_showHelp("TSR Calc Removed");
    }
    //-------------------------------------------
    // otherwise try to install it ...
    else {
        if (!InstallTSR(signature)) {
            ST_showHelp("Cannot Install TSR Calc");
        }
        else {
            ST_showHelp("TSR Calc Installed");
        }                
    }
}
